@@ -10,8 +10,10 @@ from django.shortcuts import render_to_response
 default_start_days = getattr(settings, 'DEFAULT_START_DAYS', -1000)
 default_end_days = getattr(settings, 'DEFAULT_END_DAYS', 10)
 
-DEFAULT_START = datetime.date.today() + datetime.timedelta(days=default_start_days)
-DEFAULT_END = datetime.date.today() + datetime.timedelta(days=default_end_days)
+DEFAULT_START = datetime.date.today() + datetime.timedelta(
+    days=default_start_days)
+DEFAULT_END = datetime.date.today() + datetime.timedelta(
+    days=default_end_days)
 
 DUTCH_DATE_FORMAT = '%d/%m/%Y'
 # ^^^ This is what jquery ui with the Dutch locale does for Reinout.
@@ -25,7 +27,7 @@ class DateRangeForm(forms.Form):
                                label='Einddatum')
 
 
-def set_date_range(request, template='krw_waternet/daterange.html'):
+def set_date_range(request, template='lizard_map/daterange.html'):
     """View: store the date range in the session and redirect."""
     if request.method == 'POST':
         form = DateRangeForm(request.POST)
@@ -44,7 +46,7 @@ def set_date_range(request, template='krw_waternet/daterange.html'):
 
 
 def current_start_end_dates(request, for_form=False):
-    """Return the current start/end date, either default or from session.
+    """Return the current start/end date strings, either default or from session.
 
     If for_form is True, return it as a dict so that we can pass it directly
     into a form class.  Otherwise return it as a tuple.
@@ -59,3 +61,4 @@ def current_start_end_dates(request, for_form=False):
                     date_end=date_end)
     else:
         return (date_start, date_end)
+
