@@ -2,7 +2,7 @@
 /*jslint browser: true */
 /*global $, jQuery, OpenLayers, window, map, updateLayer, fillSidebar,
 setUpScreen, nothingFoundPopup, reloadGraphs, reloadMapActions,
-setUpTransparencySlider, setUpAnimationSlider, setUpLegendTooltips */
+setUpTransparencySlider, setUpAnimationSlider, setUpTooltips */
 
 
 /*
@@ -83,16 +83,17 @@ function show_popup(data, map) {
 function hover_popup(data, map) {
     if (data.name !== "" &&
         data.name !== undefined) {
-        var size, popup, content;
+        var popup, content;
         content = '&nbsp;&nbsp;&nbsp;&nbsp;' + data.name +
             '&nbsp;&nbsp;&nbsp;&nbsp;';
         $("#hover-popup").remove(); // remove existing popup, if exists
-        size = new OpenLayers.Size(300, 80);
         popup = new OpenLayers.Popup('hover-popup',
                                      new OpenLayers.LonLat(data.x, data.y),
-                                     size,
+                                     new OpenLayers.Size(300, 35),
                                      content,
                                      false);
+        popup.maxSize = new OpenLayers.Size(300, 35);
+        popup.border = "1px solid black";
         popup.autoSize = true;
         map.addPopup(popup);
     }
@@ -247,7 +248,7 @@ jQuery.fn.updateWorkspace = function () {
                     $('.map-actions', $holder).html());
                 setUpAnimationSlider();
                 setUpTransparencySlider();
-                setUpLegendTooltips();
+                setUpTooltips();
             }
         );
     });
@@ -377,34 +378,16 @@ Then change the url (???)
 
 jQuery.fn.lizardMapLink = function () {
     $(this).click(function (event) {
-        event.preventDefault();
-        var url = event.currentTarget;
-        $.get(
-            url,
-            function (responseText, textStatus, xmlHttpRequest) {
-                //alert($(responseText).find("div#head-extras").html());
-                // replace sidebar
-                $("#sidebar").html($(responseText).find("#sidebar").html());
-                setUpScreen();
-                $(".workspace").workspaceInteraction();
-                //$(".workspace").updateWorkspaceBox();
-                // $("#head-extras").html($(responseText).find("#head-extras").html());
-
-                var headExtras = $(responseText).find("div#head-extras").html();
-                $("div#head-extras").html(headExtras);
-
-                // replace breadcrumbs
-                $("#breadcrumbs").html($(responseText).find("#breadcrumbs").html());
-                $("a.lizard-map-link").lizardMapLink(); //affects breadcrumbs AND sidebar
-
-                // replace title... where is it???
-                //console.debug($(responseText).find("title"));
-                //$("title").replaceWith($(responseText).find("title"));
-
-            }
-        );
-
-        // probeersel, je kunt niet het voorste gedeelte aanpassen
-        window.location.hash = event.currentTarget;
+        var popup_login, next;
+        popup_login = $(this).attr("data-popup-login");
+        if (popup_login !== undefined) {
+            // So we need login.
+            event.preventDefault();
+            // Fill "next" field.
+            next = $(this).attr("href");
+            $("#login-form-next").attr("value", next);
+            // "Click" on it.
+            $("#login-button").click();
+        }
     });
 };
