@@ -2,7 +2,7 @@
 /*jslint browser: true */
 /*global $, OpenLayers, window, updateLayer, updateLayers,
 stretchOneSidebarBox, reloadGraphs, fillSidebar, show_popup,
-hover_popup, layers */
+hover_popup, layers, map */
 
 var animationTimer, transparencyTimer;
 
@@ -79,6 +79,28 @@ function setUpTransparencySlider() {
 }
 
 
+function setUpMapLoadDefaultLocation() {
+    $("#map-load-default-location").click(function () {
+        var url, coordinates, zoom;
+        url = $(this).attr("data-url");
+        $.getJSON(
+            url, function (data) {
+                if ((data.x !== undefined) &&
+                    (data.y !== undefined) &&
+                    (data.zoom !== undefined))
+                {
+                    map.setCenter(
+                        new OpenLayers.LonLat(parseFloat(data.x),
+                                              parseFloat(data.y)),
+                        parseFloat(data.zoom));
+                }
+            });
+    });
+}
+
+
+/* Reload map-actions: put all initialization functions here for map actions
+(above map, load/save location/ empty temp workspace) */
 function reloadMapActions() {
     $(".map-actions").load(
         "./ .map-action",
@@ -86,6 +108,7 @@ function reloadMapActions() {
             fillSidebar();
             setUpAnimationSlider();
             setUpTransparencySlider();
+            setUpMapLoadDefaultLocation();
         });
 }
 
@@ -123,13 +146,84 @@ function setUpWorkspaceAcceptable() {
              adapter_layer_json: adapter_layer_json
             },
             function (workspace_id) {
+                var url, center_x, center_y;
                 updateLayer(workspace_id);
                 reloadMapActions();
+                // Load extent from workspace and zoom to it.
+                url = $(".workspace").attr(
+                    "data-url-lizard-map-session-workspace-extent-temp");
+                $.getJSON(url, {}, function (extent) {
+                    // If we do not get extent, just forget it.
+                    if ((extent.north !== null) &&
+                        (extent.south !== null) &&
+                        (extent.east !== null) &&
+                        (extent.west !== null))
+                    {
+                        // Convert bbox to center coordinates
+                        center_x = (extent.east + extent.west) / 2;
+                        center_y = (extent.north + extent.south) / 2;
+                        // Now pan
+                        map.panTo(
+                            new OpenLayers.LonLat(parseFloat(center_x),
+                                                  parseFloat(center_y)));
+                    }
+                });
             });
         stretchOneSidebarBox();
     });
+
+    // Add action to button 'add-workspace-item' in workspace
+    // acceptables. This class is added by
+    // setUpWorkspaceAcceptableButtons in lizard-ui: lizard.js.
+    $(".add-workspace-item").live("click", function (event) {
+        // Fetch workspace acceptable properties and add workspace item.
+        var wa_name, adapter_class, adapter_layer_json, url,
+            $workspace_acceptable, $workspace, workspace_id;
+        // Get adapter class and parameters.
+        $workspace_acceptable = $(this).parents(".workspace-acceptable");
+        $workspace = $(".workspace");  // Assumes there is only 1.
+        workspace_id = $workspace.attr("data-workspace-id");
+
+        wa_name = $workspace_acceptable.attr("data-name");
+        adapter_class = $workspace_acceptable.attr("data-adapter-class");
+        adapter_layer_json = $workspace_acceptable.attr("data-adapter-layer-json");
+        url = $workspace.attr("data-url-lizard-map-workspace-item-add");
+
+        // Request to make workspace item and update workspace.
+        $.post(
+            url,
+            {workspace_id: workspace_id,
+             name: wa_name,
+             adapter_class: adapter_class,
+             adapter_layer_json: adapter_layer_json
+            },
+            function (workspace_id) {
+                // Update all workspaces.
+                $(".workspace").updateWorkspace();
+                // "click" the empty temp workspace
+                $(".workspace-empty-temp").click();
+            }
+        );
+        return false; // Same as .preventDefault and .stopPropagation
+    });
 }
 
+
+function setUpWorkspaceEmpty() {
+    var $workspace, workspace_id, url;
+    $(".workspace-empty-trigger").live('click', function () {
+        $workspace = $(this).parents("div.workspace");
+        workspace_id = $workspace.attr("data-workspace-id");
+        url = $workspace.attr("data-url-lizard-map-workspace-item-empty");
+        $.post(
+            url, {workspace_id: workspace_id},
+            function (data) {
+                $workspace.updateWorkspace();
+            });
+    });
+}
+
+
 // Date selector: only support for ONE date selector at the moment.
 
 function setUpDateChoice() {
@@ -195,38 +289,6 @@ function setUpGraphEditPopup() {
 }
 
 
-/*
-Shows legend tooltip. Re-initializes after workspace update.
-*/
-function setUpLegendTooltips() {
-    $(".legend-tooltip").each(function () {
-        if (!$(this).data("popup-initialized")) {
-            $(this).data("popup-initialized", true);
-            $(this).tooltip({
-                position: 'center right',
-                effect: 'fade',
-                onShow: function () {
-                    var offset, pixels_below_screen;
-                    // Too high?
-                    offset = this.getTip().offset();
-                    if (offset.top < 0) {
-                        offset.top = 0;
-                        this.getTip().offset(offset);
-                    }
-                    // Too low?
-                    pixels_below_screen = offset.top +
-                        this.getTip().height() -
-                        $(window).height();
-                    if (pixels_below_screen > 0) {
-                        offset.top = offset.top - pixels_below_screen;
-                        this.getTip().offset(offset);
-                    }
-                    // Repositioning beforehand would be visually nicer.
-                }
-            });
-        }
-    });
-}
 
 
 
@@ -293,22 +355,7 @@ function popup_hover_handler(x, y, map) {
 }
 
 
-function legend_action_reload(event) {
-    // send all legend properties to server and reload page
-    var $form, url, name;
-    event.preventDefault();
-    $form = $(this).parents("form.legend-options");
-    url = $form.attr("data-url");
-    $.post(
-        url,
-        $form.serialize(),
-        function () {
-            // Reload page after posting.
-            location.reload();
-        });
-}
-
-
+/* Legend edit functions. */
 function setUpLegendColorPickers() {
     var submit, beforeshow;
     submit = function (hsb, hex, rgb, el) {
@@ -355,6 +402,55 @@ function setUpLegendColorPickers() {
 }
 
 
+/* Pans to workspace item. Only works if extent function is
+implemented for that particilar workspace item. */
+function setUpWorkspaceItemPanToLayer() {
+    $(".workspace-item-pan-to").live("click", function () {
+        var url, workspace_item_id;
+        workspace_item_id = $(this).parent().attr(
+            "data-object-id");
+        url = $(this).parents(".workspace").attr(
+            "data-url-lizard-map-workspace-item-extent");
+        $.getJSON(
+            url,
+            {workspace_item_id: workspace_item_id},
+            function (extent) {
+                var center_x, center_y;
+                // If we do not get extent, just forget it.
+                if ((extent.north !== null) &&
+                    (extent.south !== null) &&
+                    (extent.east !== null) &&
+                    (extent.west !== null))
+                {
+                    // Convert bbox to center coordinates
+                    center_x = (extent.east + extent.west) / 2;
+                    center_y = (extent.north + extent.south) / 2;
+                    // Now pan
+                    map.panTo(
+                        new OpenLayers.LonLat(parseFloat(center_x),
+                                              parseFloat(center_y)));
+                }
+            });
+    });
+}
+
+
+function legend_action_reload(event) {
+    // send all legend properties to server and reload page
+    var $form, url, name;
+    event.preventDefault();
+    $form = $(this).parents("form.legend-options");
+    url = $form.attr("data-url");
+    $.post(
+        url,
+        $form.serialize(),
+        function () {
+            // Reload page after posting.
+            location.reload();
+        });
+}
+
+
 function setUpLegendEdit() {
     $(".legend-edit").live("mouseover", function () {
         if (!$(this).data("popup-initialized")) {
@@ -367,9 +463,24 @@ function setUpLegendEdit() {
 }
 
 
+function mapSaveLocation() {
+    var url, coordinates;
+    url = $("#lizard-map-wms").attr("data-save-location-url");
+    coordinates = map.center;
+    $.ajax({
+        type: 'POST',
+        url: url,
+        data: {x: coordinates.lon, y: coordinates.lat, zoom: map.zoom},
+        async: false,
+        success: function () {}
+    });
+}
+
+
 // Initialize all workspace actions.
 $(document).ready(function () {
     setUpWorkspaceAcceptable();
+    setUpWorkspaceEmpty();
     setUpDatePopup();
     setUpDateChoice();
     setUpNotFoundPopup();
@@ -377,13 +488,22 @@ $(document).ready(function () {
     setUpAnimationSlider();
     setUpTransparencySlider();
     setUpGraphEditPopup();
+    setUpWorkspaceItemPanToLayer();
 
-    // Set up legend.
-    setUpLegendTooltips(); // The edit function is on the tooltip.
+    // Set up legend edit.
     setUpLegendEdit();
 
+    setUpMapLoadDefaultLocation();
+
     /* Workspace functions, requires jquery.workspace.js */
     $(".workspace").workspaceInteraction();
     $(".add-snippet").snippetInteraction(); // voor collage view, nu nog nutteloos voor popup
-    // $("a.lizard-map-link").lizardMapLink();
+    $("a.lizard-map-link").lizardMapLink();
+});
+
+// Beforeunload: this function is called just before leaving the page
+// and loading the new page. Unload however is called after loading
+// the new page.
+$(window).bind('beforeunload', function () {
+    mapSaveLocation(); // Save map location when 'before' leaving page.
 });
