@@ -1,11 +1,13 @@
 import itertools
 import logging
+import mapnik
 
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
 from django.db import models
 from django.utils import simplejson as json
 from django.utils.translation import ugettext as _
+from south.modelsinspector import add_introspection_rules
 import pkg_resources
 
 from lizard_map.adapter import parse_identifier_json
@@ -17,6 +19,7 @@ from lizard_map.dateperiods import WEEK
 from lizard_map.dateperiods import DAY
 from lizard_map.dateperiods import calc_aggregation_periods
 from lizard_map.dateperiods import fancy_period
+from lizard_map.mapnik_helper import point_rule
 
 # Do not change the following items!
 GROUPING_HINT = 'grouping_hint'
@@ -24,12 +27,121 @@ USER_WORKSPACES = 'user'
 DEFAULT_WORKSPACES = 'default'
 TEMP_WORKSPACES = 'temp'
 
+# TODO: Can this property be moved to mapnik_helper?
 ICON_ORIGINALS = pkg_resources.resource_filename('lizard_map', 'icons')
+
 ADAPTER_ENTRY_POINT = 'lizard_map.adapter_class'
 SEARCH_ENTRY_POINT = 'lizard_map.search_method'
 LOCATION_ENTRY_POINT = 'lizard_map.location_method'
 
-logger = logging.getLogger('lizard_map.models')
+logger = logging.getLogger(__name__)
+# Add introspection rules for ColorField
+add_introspection_rules([], ["lizard_map.models.ColorField"])
+
+
+def legend_values(min_value, max_value, min_color, max_color, steps):
+    """Interpolates colors between min_value and max_value, calc
+    corresponding colors and gives boundary values for each band.
+
+    Makes list of dictionaries: {'color': Color, 'low_value':
+    low value, 'high_value': high value}"""
+    result = []
+    value_per_step = (max_value - min_value) / steps
+    for step in range(steps):
+        try:
+            fraction = float(step) / (steps - 1)
+        except ZeroDivisionError:
+            fraction = 0
+        alpha = (min_color.a * (1 - fraction) +
+                 max_color.a * fraction)
+        red = (min_color.r * (1 - fraction) +
+               max_color.r * fraction)
+        green = (min_color.g * (1 - fraction) +
+                 max_color.g * fraction)
+        blue = (min_color.b * (1 - fraction) +
+                max_color.b * fraction)
+        color = Color('%02x%02x%02x%02x' % (red, green, blue, alpha))
+
+        low_value = min_value + step * value_per_step
+        high_value = min_value + (step + 1) * value_per_step
+        result.append({
+                'color': color,
+                'low_value': low_value,
+                'high_value': high_value,
+                })
+    return result
+
+
+class Color(str):
+    """Simple color object: r, g, b, a.
+
+    The object is in fact a string with class variables.
+    """
+    def __init__(self, s):
+        if s is None:
+            self.r = None
+            self.g = None
+            self.b = None
+            return
+        try:
+            self.r = int(s[0:2], 16)
+        except ValueError:
+            self.r = 128
+        try:
+            self.g = int(s[2:4], 16)
+        except ValueError:
+            self.b = 128
+        try:
+            self.b = int(s[4:6], 16)
+        except ValueError:
+            self.b = 128
+        try:
+            # Alpha is optional.
+            self.a = int(s[6:8], 16)
+        except ValueError:
+            self.a = 255
+
+    def to_tuple(self):
+        """
+        Returns color values in a tuple. Values are 0..1
+        """
+        result = (self.r / 255.0, self.g / 255.0,
+                  self.b / 255.0, self.a / 255.0)
+        return result
+
+    @property
+    def html(self):
+        """
+        Returns color in html format.
+        """
+        return '#%02x%02x%02x' % (self.r, self.g, self.b)
+
+
+class ColorField(models.CharField):
+    """Custom ColorField for use in Django models. It's an extension
+    of CharField."""
+
+    default_error_messages = {
+        'invalid': _(
+            u'Enter a valid color code rrggbbaa, '
+            'where aa is optional.'),
+        }
+    description = "Color representation in rgb"
+
+    # Ensures that to_python is always called.
+    __metaclass__ = models.SubfieldBase
+
+    def __init__(self, *args, **kwargs):
+        kwargs['max_length'] = 8
+        super(ColorField, self).__init__(*args, **kwargs)
+
+    def get_prep_value(self, value):
+        return str(value)
+
+    def to_python(self, value):
+        if isinstance(value, Color):
+            return value
+        return Color(value)
 
 
 def adapter_class_names():
@@ -55,12 +167,8 @@ class Workspace(models.Model):
 
     name = models.CharField(max_length=80,
                             blank=True,
-                            default='Workspace')
-    # TODO below: default extend values for NL?
-    extent_north = models.FloatField(blank=True, null=True)
-    extent_east = models.FloatField(blank=True, null=True)
-    extent_south = models.FloatField(blank=True, null=True)
-    extent_west = models.FloatField(blank=True, null=True)
+                            default='My Workspace')
+
     owner = models.ForeignKey(User, blank=True, null=True)
     visible = models.BooleanField(default=False)
 
@@ -71,6 +179,25 @@ class Workspace(models.Model):
         return reverse('lizard_map_workspace',
                        kwargs={'workspace_id': self.id})
 
+    def extent(self):
+        """
+        Returns workspace extent, using extents from workspace items.
+        """
+        north = None
+        south = None
+        east = None
+        west = None
+        for workspace_item in self.workspace_items.all():
+            wsi_extent = workspace_item.adapter.extent()
+            if wsi_extent['east'] > east or east is None:
+                east = wsi_extent['east']
+            if wsi_extent['west'] < west or west is None:
+                west = wsi_extent['west']
+            if wsi_extent['south'] < south or south is None:
+                south = wsi_extent['south']
+            if wsi_extent['north'] > north or north is None:
+                north = wsi_extent['north']
+        return {'north': north, 'south': south, 'east': east, 'west': west}
 
 class WorkspaceItem(models.Model):
     """Can show things on a map based on configuration in a url."""
@@ -317,10 +444,14 @@ class WorkspaceCollageSnippetGroup(models.Model):
         if len(statistics) > 1:
             # Also show a 'totals' column.
             averages = [row['avg'] for row in statistics if row['avg']]
+            try:
+                average = float(sum(averages) / len(averages))
+            except ZeroDivisionError:
+                average = None
             totals = {
                 'min': min([row['min'] for row in statistics]),
                 'max': max([row['max'] for row in statistics]),
-                'avg': float(sum(averages) / len(averages)),
+                'avg': average,
                 'name': 'Totaal',
                 }
             if statistics[0]['count_lt'] is not None:
@@ -494,20 +625,6 @@ class WorkspaceCollageSnippet(models.Model):
         self.identifier_json = json.dumps(identifier).replace('"', '%22')
 
 
-class Color(models.Model):
-    """Simple color: alpha (optional), r, g, b.
-
-    Use values from 0 .. 255. Values are NOT checked!
-    """
-    a = models.IntegerField(default=255)
-    r = models.IntegerField()
-    g = models.IntegerField()
-    b = models.IntegerField()
-
-    def __unicode__(self):
-        return '(%0x)-%0x-%0x-%0x' % (self.a, self.r, self.g, self.b)
-
-
 class LegendManager(models.Manager):
     """Implements extra function 'find'
     """
@@ -529,17 +646,21 @@ class LegendManager(models.Manager):
 class Legend(models.Model):
     """Simple lineair interpolated legend with min, max, color-min,
     color-max, color < min, color > max, number of steps. Legends can
-    be found using the descriptor"""
+    be found using the descriptor.
+
+    Used for mapnik lines and polygons.
+    """
 
     descriptor = models.CharField(max_length=80)
     min_value = models.FloatField(default=0)
     max_value = models.FloatField(default=100)
     steps = models.IntegerField(default=10)
 
-    min_color = models.ForeignKey(Color, related_name='min_colors')
-    max_color = models.ForeignKey(Color, related_name='max_colors')
-    too_low_color = models.ForeignKey(Color, related_name='too_low_colors')
-    too_high_color = models.ForeignKey(Color, related_name='too_high_color')
+    default_color = ColorField()
+    min_color = ColorField()
+    max_color = ColorField()
+    too_low_color = ColorField()
+    too_high_color = ColorField()
 
     objects = LegendManager()
 
@@ -548,6 +669,7 @@ class Legend(models.Model):
 
     @property
     def float_format(self):
+        """Determines float format for defined legend. Required by legend_default."""
         delta = abs(self.max_value - self.min_value) / self.steps
         if delta < 0.1:
             return '%.3f'
@@ -558,33 +680,10 @@ class Legend(models.Model):
         return '%.0f'
 
     def legend_values(self):
-        """Makes list of dictionaries: {'color': Color, 'low_value':
-        low value, 'high_value': high value}"""
-        result = []
-        value_per_step = (self.max_value - self.min_value) / self.steps
-        for step in range(self.steps):
-            try:
-                fraction = float(step) / (self.steps - 1)
-            except ZeroDivisionError:
-                fraction = 0
-            alpha = (self.min_color.a * (1 - fraction) +
-                     self.max_color.a * fraction)
-            red = (self.min_color.r * (1 - fraction) +
-                   self.max_color.r * fraction)
-            green = (self.min_color.g * (1 - fraction) +
-                     self.max_color.g * fraction)
-            blue = (self.min_color.b * (1 - fraction) +
-                    self.max_color.b * fraction)
-            color = Color(a=alpha, r=red, g=green, b=blue)
-
-            low_value = self.min_value + step * value_per_step
-            high_value = self.min_value + (step + 1) * value_per_step
-            result.append({
-                    'color': color,
-                    'low_value': low_value,
-                    'high_value': high_value,
-                    })
-        return result
+        """Determines legend steps and values. Required by legend_default."""
+        return legend_values(
+            self.min_value, self.max_value,
+            self.min_color, self.max_color, self.steps)
 
     def update(self, updates):
         """ Updates model with updates dict. Color values have the
@@ -622,3 +721,117 @@ class Legend(models.Model):
                     self.too_high_color = makecolor(v)
                 except ValueError:
                     logger.warn('Could not parse too_high_color (%s)' % v)
+
+    def mapnik_style(self, value_field=None):
+        """Return a Mapnik line/polystyle from Legend object"""
+
+        def mapnik_rule(color, mapnik_filter=None):
+            """
+            Makes mapnik rule for looks. For lines and polygons.
+            """
+            rule = mapnik.Rule()
+            if mapnik_filter is not None:
+                rule.filter = mapnik.Filter(mapnik_filter)
+            mapnik_color = mapnik.Color(color.r, color.g, color.b)
+
+            symb_line = mapnik.LineSymbolizer(mapnik_color, 3)
+            rule.symbols.append(symb_line)
+
+            symb_poly = mapnik.PolygonSymbolizer(mapnik_color)
+            symb_poly.fill_opacity = 0.5
+            rule.symbols.append(symb_poly)
+            return rule
+
+        mapnik_style = mapnik.Style()
+        if value_field is None:
+            value_field = "value"
+
+        # Default color: red.
+        rule = mapnik_rule(self.default_color)
+        mapnik_style.rules.append(rule)
+
+        # < min
+        mapnik_filter = "[%s] <= %f" % (value_field, self.min_value)
+        logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+        rule = mapnik_rule(self.too_low_color, mapnik_filter)
+        mapnik_style.rules.append(rule)
+
+        # in boundaries
+        for legend_value in self.legend_values():
+            mapnik_filter = "[%s] > %f and [%s] <= %f" % (
+                value_field, legend_value['low_value'],
+                value_field, legend_value['high_value'])
+            logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+            rule = mapnik_rule(legend_value['color'], mapnik_filter)
+            mapnik_style.rules.append(rule)
+
+        # > max
+        mapnik_filter = "[%s] > %f" % (value_field, self.max_value)
+        logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+        rule = mapnik_rule(self.too_high_color, mapnik_filter)
+        mapnik_style.rules.append(rule)
+
+        return mapnik_style
+
+    def mapnik_linestyle(self, value_field=None):
+        """Deprecated. Use mapnik_style instead. Return a Mapnik
+        line/polystyle from Legend object"""
+        return mapnik_style(self, value_field=value_field)
+
+
+class LegendPoint(Legend):
+    """
+    Legend for points.
+    """
+
+    icon = models.CharField(max_length=80, default='empty.png')
+    mask = models.CharField(max_length=80, null=True, blank=True,
+                            default='empty_mask.png')
+
+    def __unicode__(self):
+        return '%s' % (self.descriptor)
+
+    def icon_style(self):
+        return {'icon': self.icon,
+                'mask': (self.mask, ),
+                'color': (1.0, 1.0, 1.0, 1.0)}
+
+    def mapnik_style(self, value_field=None):
+        """Return a Mapnik style from Legend object. """
+
+        mapnik_style = mapnik.Style()
+        if value_field is None:
+            value_field = "value"
+
+        # Default color: red.
+        mapnik_rule = point_rule(
+            self.icon, self.mask, self.default_color)
+        mapnik_style.rules.append(mapnik_rule)
+
+        # < min
+        mapnik_filter = "[%s] <= %f" % (value_field, self.min_value)
+        logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+        mapnik_rule = point_rule(
+            self.icon, self.mask, self.too_low_color, mapnik_filter=mapnik_filter)
+        mapnik_style.rules.append(mapnik_rule)
+
+        # in boundaries
+        for legend_value in self.legend_values():
+            mapnik_filter = "[%s] > %f and [%s] <= %f" % (
+                value_field, legend_value['low_value'],
+                value_field, legend_value['high_value'])
+            logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+            mapnik_rule = point_rule(
+                self.icon, self.mask, legend_value['color'],
+                mapnik_filter=mapnik_filter)
+            mapnik_style.rules.append(mapnik_rule)
+
+        # > max
+        mapnik_filter = "[%s] > %f" % (value_field, self.max_value)
+        logger.debug('adding mapnik_filter: %s' % mapnik_filter)
+        mapnik_rule = point_rule(
+            self.icon, self.mask, self.too_high_color,
+            mapnik_filter=mapnik_filter)
+        mapnik_style.rules.append(mapnik_rule)
+
+        return mapnik_style
