@@ -3,6 +3,7 @@ import datetime
 from django.core.urlresolvers import reverse
 from django.test import TestCase
 from django.test.client import Client
+from django.utils import simplejson as json
 import pkg_resources
 
 from lizard_map.adapter import Graph
@@ -18,35 +19,32 @@ from lizard_map.dateperiods import WEEK
 from lizard_map.dateperiods import DAY
 from lizard_map.dateperiods import calc_aggregation_periods
 from lizard_map.dateperiods import fancy_period
-from lizard_map.layers import WorkspaceItemAdapterShapefile
+from lizard_map.mapnik_helper import database_settings
+from lizard_map.models import Color
+from lizard_map.models import Legend
 from lizard_map.models import Workspace
 from lizard_map.models import WorkspaceCollage
 from lizard_map.models import WorkspaceCollageSnippetGroup
 from lizard_map.models import WorkspaceItem
+from lizard_map.operations import AnchestorRegistration
+from lizard_map.operations import CycleError
+from lizard_map.operations import named_list
+from lizard_map.operations import tree_from_list
+from lizard_map.operations import unique_list
+from lizard_map.utility import float_to_string
 from lizard_map.utility import short_string
 from lizard_map.workspace import WorkspaceItemAdapter
 from lizard_map.workspace import WorkspaceManager
+from lizard_map.templatetags import map as map_functions
 from lizard_map.templatetags import workspaces
 import lizard_map.admin
+import lizard_map.coordinates
 import lizard_map.layers
 import lizard_map.models
 import lizard_map.urls
 import lizard_map.views
 
 
-class LayersTest(TestCase):
-    """WMS layer functions are generally defined in layers.py. One can add his
-    own in other apps. shapefile_layer is an example function.
-    """
-
-    def test_initialization(self):
-        mock_workspace = None
-        ws_adapter = lizard_map.layers.WorkspaceItemAdapterShapefile(
-            mock_workspace, layer_arguments={})
-        layers, styles = ws_adapter.layer()
-        # TODO: test output.
-
-
 class WorkspaceManagerTest(TestCase):
 
     class MockRequest(object):
@@ -137,6 +135,27 @@ class ViewsTest(TestCase):
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
 
+    def test_map_location_save(self):
+        """Save map location, see if it doesn't crash
+        """
+        client = Client()
+        url = reverse('lizard_map.map_location_save')
+        response = client.post(url, {'x': 100, 'y': 150, 'zoom': 10})
+        self.assertEqual(response.status_code, 200)
+
+    def test_map_location_load_default(self):
+        """Save map location, then load it back.
+        """
+        url_load = reverse('lizard_map.map_location_load_default')
+        response_load = self.client.get(url_load)
+        self.assertEqual(response_load.status_code, 200)
+        result = json.loads(response_load.content)
+        self.assertEqual(
+            result,
+            {'x': '550000',
+             'y': '6850000',
+             'zoom': '10'})
+
 
 class WorkspaceTest(TestCase):
 
@@ -153,9 +172,10 @@ class WorkspaceTest(TestCase):
         self.assertTrue(workspace_item2 in workspace1.workspace_items.all())
 
     def test_name(self):
-        """A workspace always has a name.  It is 'workspace' by default."""
+        """A workspace always has a name.  It is 'My Workspace' by
+        default."""
         workspace = Workspace()
-        self.assertEquals(workspace.name, u'Workspace')
+        self.assertEquals(workspace.name, u'My Workspace')
 
     def test_representation(self):
         workspace = Workspace()
@@ -236,6 +256,32 @@ class WorkspaceClientTest(TestCase):
             {'object_id': str(workspace_item1.id)})
         self.assertFalse(workspace.workspace_items.all())
 
+    def test_workspace_extent_temp(self):
+        """
+        Tests if the workspace extent does not crash (although the
+        content will be meaningless).
+        """
+        url = reverse('lizard_map_session_workspace_extent_temp')
+        result = self.client.get(url, {})
+        self.assertEqual(result.status_code, 200)
+
+    # Not testable without adapter
+    # def test_workspace_extent(self):
+    #     """
+    #     Tests if the workspace extent does not crash (although the
+    #     content will be meaningless).
+    #     """
+    #     workspace = Workspace()
+    #     workspace.save()
+    #     workspace_item1 = workspace.workspace_items.create(
+    #         name='test workspaceitem')
+    #     url = reverse('lizard_map_workspace_item_extent')
+    #     result = self.client.get(url, {
+    #             'workspace_item_id': workspace_item1.id})
+    #     self.assertEqual(result.status_code, 200)
+    #     self.assertTrue('north' in result.content)
+
+
     # Not testable without adapter
     # def test_workspace_item_image(self):
     #     workspace = Workspace()
@@ -275,10 +321,11 @@ class WorkspaceItemTest(TestCase):
                           # ^^^ lambda as adapter is a property and
                           # assertRaises expects a callable.
                           )
-        workspace_item.adapter_class = 'adapter_shapefile'
+        workspace_item.adapter_class = 'adapter_dummy'
+        workspace_item.adapter_layer_json = ("{}")
         self.assertTrue(isinstance(
                 workspace_item.adapter,
-                lizard_map.layers.WorkspaceItemAdapterShapefile))
+                lizard_map.layers.AdapterDummy))
 
     def test_adapter_arguments(self):
         """The layer method probably needs arguments. You can store them as a
@@ -408,8 +455,10 @@ class TestAnimationSettings(TestCase):
 
 class UtilityTest(TestCase):
     """
-    Tests utility.py & adapter.py
+    Tests utility.py, adapter.py, mapnik_helper.py
     """
+    class MockSettings(object):
+        pass
 
     def test_short_string(self):
         input_names = [
@@ -423,6 +472,47 @@ class UtilityTest(TestCase):
             self.assertTrue(len(short) <= 17)
             self.assertEquals(short[:5], name[:5])
 
+    def test_float_to_string(self):
+        """Let's input some nice floats."""
+        st = float_to_string(1.2345678)
+        self.assertTrue(len(st) <= 10)
+        self.assertEquals(st[:4], '1.23')
+
+    def test_float_to_string2(self):
+        """Let's input some nice floats."""
+        st = float_to_string(12345.678)
+        self.assertTrue(len(st) <= 10)
+        self.assertEquals(st[:4], '1234')
+
+    def test_float_to_string3(self):
+        """Let's input some nice floats."""
+        st = float_to_string(0.00000000012345)
+        self.assertTrue(len(st) <= 10)
+        self.assertEquals(st[:4], '1.23')
+        self.assertTrue('e' in st)
+
+    def test_float_to_string5(self):
+        """Let's input some nice floats."""
+        st = float_to_string(123456789012345)
+        self.assertTrue(len(st) <= 10)
+        self.assertEquals(st[:4], '1.23')
+        self.assertTrue('e' in st)
+
+    def test_float_to_string6(self):
+        """Let's input some nice floats."""
+        st = float_to_string(999999999.99)
+        self.assertTrue(len(st) <= 10)
+        self.assertEquals(st[:4], '1000')
+
+    def test_float_to_string7(self):
+        """Let's input a string."""
+        self.assertEquals(float_to_string('string'), 'string')
+
+    def test_float_to_string8(self):
+        """Let's input a string."""
+        st = float_to_string('1000')
+        self.assertEquals(st, '1000.00')
+
     def test_parse_identifier_json(self):
         self.assertTrue(parse_identifier_json(
                 '{%22testkey%22:%20%22testvalue%22}'))
@@ -443,6 +533,97 @@ class UtilityTest(TestCase):
         graph.legend()
         self.assertTrue(graph.http_png())
 
+    def test_database_settings(self):
+        """
+        See if correct database settings are fetched when using new
+        style config.
+        """
+        settings = self.MockSettings()
+        settings.DATABASES = {
+            'default': {
+                'ENGINE': 'postgresql_psycopg2',
+                'HOST': 'database_host',
+                'USER': 'database_user',
+                'PASSWORD': 'database_password',
+                'NAME': 'database_name',
+                },
+            }
+        datasource, options = database_settings(user_settings=settings)
+        self.assertEqual(options,
+                         {'host': 'database_host',
+                          'user': 'database_user',
+                          'password': 'database_password',
+                          'dbname': 'database_name'})
+        self.assertTrue('PostGIS' in str(datasource))
+
+    def test_database_settings2(self):
+        """
+        See if correct database settings are fetched when using old
+        style config.
+        """
+        settings = self.MockSettings()
+        settings.DATABASE_ENGINE = 'postgresql_psycopg2'
+        settings.DATABASE_NAME = 'database_name'
+        settings.DATABASE_HOST = 'database_host'
+        settings.DATABASE_USER = 'database_user'
+        settings.DATABASE_PASSWORD = 'database_password'
+
+        datasource, options = database_settings(user_settings=settings)
+        self.assertEqual(options,
+                         {'host': 'database_host',
+                          'user': 'database_user',
+                          'password': 'database_password',
+                          'dbname': 'database_name'})
+        self.assertTrue('PostGIS' in str(datasource))
+
+    def test_database_settings3(self):
+        """
+        Take new style config if both setting styles are present.
+        """
+        settings = self.MockSettings()
+        settings.DATABASE_ENGINE = 'postgresql_psycopg2'
+        settings.DATABASE_NAME = 'database_name_old'
+        settings.DATABASE_HOST = 'database_host_old'
+        settings.DATABASE_USER = 'database_user_old'
+        settings.DATABASE_PASSWORD = 'database_password_old'
+        settings.DATABASES = {
+            'default': {
+                'ENGINE': 'postgresql_psycopg2',
+                'HOST': 'database_host',
+                'USER': 'database_user',
+                'PASSWORD': 'database_password',
+                'NAME': 'database_name',
+                },
+            }
+
+        datasource, options = database_settings(user_settings=settings)
+        self.assertEqual(options,
+                         {'host': 'database_host',
+                          'user': 'database_user',
+                          'password': 'database_password',
+                          'dbname': 'database_name'})
+        self.assertTrue('PostGIS' in str(datasource))
+
+    # def test_create_layer_from_query(self):
+    #     """Difficult to test, just see if it crashes or not. Currently
+    #     crashes when db does not exist
+    #     """
+    #     settings = self.MockSettings()
+    #     settings.DATABASES = {
+    #         'default': {
+    #             'ENGINE': 'postgresql_psycopg2',
+    #             'HOST': 'database_host',
+    #             'USER': 'database_user',
+    #             'PASSWORD': 'database_password',
+    #             'NAME': 'database_name',
+    #             },
+    #         }
+
+    #     q = "select * from dummy;"
+    #     projection = lizard_map.coordinates.RD
+    #     layer = create_layer_from_query(q, projection,
+    #                 user_settings=settings)
+
 
 class WorkspaceItemAdapterTest(TestCase):
 
@@ -502,6 +683,13 @@ class WorkspaceItemAdapterTest(TestCase):
         snippet_group.save()
         self.assertTrue(self.adapter.html_default(snippet_group=snippet_group))
 
+    def test_legend_object_default(self):
+        self.adapter.legend_object_default('no_name')
+
+    def test_legend_default(self):
+        legend_object = self.adapter.legend_object_default('no_name')
+        self.adapter.legend_default(legend_object)
+
 
 class DatePeriodsTest(TestCase):
 
@@ -567,40 +755,6 @@ class DatePeriodsTest(TestCase):
         self.assertTrue(fancy_period(start_date, end_date, WEEK))
         self.assertTrue(fancy_period(start_date, end_date, DAY))
 
-class WorkspaceItemAdapterShapefileTestSuite(TestCase):
-
-    def test_a(self):
-        """Test the default layer info is initialized correctly."""
-
-        workspace_item = 0 # don't care for this test
-        adapter = WorkspaceItemAdapterShapefile(workspace_item)
-
-        layer_name = lizard_map.layers.default_layer_name
-        resource_module = lizard_map.layers.default_resource_module
-        resource_name = lizard_map.layers.default_resource_name
-        search_property_name = lizard_map.layers.default_search_property_name
-
-        self.assertEqual(adapter.layer_name, layer_name)
-        self.assertEqual(adapter.resource_module, resource_module)
-        self.assertEqual(adapter.resource_name, resource_name)
-        self.assertEqual(adapter.search_property_name, search_property_name)
-
-    def test_b(self):
-        """Test the layer info is initialized with the given parameters."""
-
-        workspace_item = 0 # don't care for this test
-        arguments = {'layer_name': 'Layer name',
-                     'resource_module': 'Resource module',
-                     'resource_name': 'Resource name',
-                     'search_property_name': 'Search property name'}
-
-        adapter = WorkspaceItemAdapterShapefile(workspace_item,
-                                                layer_arguments=arguments)
-
-        self.assertEqual(adapter.layer_name, 'Layer name')
-        self.assertEqual(adapter.resource_module, 'Resource module')
-        self.assertEqual(adapter.resource_name, 'Resource name')
-        self.assertEqual(adapter.search_property_name, 'Search property name')
 
 class TestTemplateTags(TestCase):
 
@@ -617,3 +771,207 @@ class TestTemplateTags(TestCase):
                   }
         for value, expected in in_out.items():
             self.assertEquals(workspaces.float_or_exp(value), expected)
+
+    def test_map_variables(self):
+        """Just see if map_variables crashes"""
+        parser = None
+        token = None
+        map_functions.map_variables(parser, token)
+
+    def test_detect_browser(self):
+        """Just see if detect_browser crashes"""
+        parser = None
+        token = None
+        map_functions.detect_browser(parser, token)
+
+
+class ModelLegendTest(TestCase):
+
+    def test_make_legend(self):
+        legend = Legend(
+            descriptor='default',
+            min_color='ff0000',
+            max_color='0000ff',
+            too_low_color='111111',
+            too_high_color='999999')
+        legend.save()
+        self.assertEquals(legend.min_color.r, 255)
+        self.assertEquals(legend.max_color.r, 0)
+        self.assertEquals(legend.max_color.b, 255)
+
+    def test_color(self):
+        """Testing the color object"""
+        c = Color('ff8000')
+        self.assertEquals(c.r, 255)
+        self.assertEquals(c.g, 128)
+        self.assertEquals(c.b, 0)
+
+    def test_color2(self):
+        c = Color(None)
+        self.assertEquals(c.r, None)
+        self.assertEquals(c.g, None)
+        self.assertEquals(c.b, None)
+
+
+class TestOperations(TestCase):
+
+    def test_anchestor_registration(self):
+        """Check basic functionality"""
+        anchestors = AnchestorRegistration()
+        self.assertFalse(anchestors.anchestor_of('child', 'parent'))
+        anchestors.register_parent('child', 'parent')
+        self.assertTrue(anchestors.anchestor_of('child', 'parent'))
+        anchestors.register_parent('grandchild', 'child')
+        self.assertTrue(anchestors.anchestor_of('grandchild', 'parent'))
+        self.assertFalse(anchestors.anchestor_of('parent', 'grandchild'))
+
+    def test_anchestor_registration2(self):
+        """The anchestor registration doesn't mind cycles"""
+        anchestors = AnchestorRegistration()
+        anchestors.register_parent('child', 'parent')
+        anchestors.register_parent('parent', 'child')
+        self.assertTrue(anchestors.anchestor_of('child', 'parent'))
+        self.assertTrue(anchestors.anchestor_of('parent', 'child'))
+
+    def test_tree_from_list1(self):
+        rows = []
+        result_good = []
+        result_function = tree_from_list(rows)
+        self.assertEqual(result_function, result_good)
+
+    def test_tree_from_list2(self):
+        rows = [{'name': 'parent_name', 'parent': None},
+                {'name': 'child_name', 'parent': 'parent_name'}]
+        result_good = [
+                {'name': 'parent_name',
+                 'parent': None,
+                 'children': [
+                    {'name': 'child_name',
+                     'parent': 'parent_name',
+                     'children': []}]}]
+        result_function = tree_from_list(
+            rows,
+            id_field='name',
+            parent_field='parent',
+            children_field='children',
+            root_parent=None)
+        self.assertEqual(result_function, result_good)
+
+    def test_tree_from_list3(self):
+        rows = [{'name': 'parent_name', 'parent': None},
+                {'name': 'child_name', 'parent': 'parent_name'},
+                {'name': 'child_name2', 'parent': 'parent_name'},
+                {'name': 'child_name3', 'parent': 'parent_name'},
+                {'name': 'child_child', 'parent': 'child_name3'}]
+        result_good = [
+                {'name': 'parent_name',
+                 'parent': None,
+                 'children': [
+                    {'name': 'child_name',
+                     'parent': 'parent_name',
+                     'children': []},
+                    {'name': 'child_name2',
+                     'parent': 'parent_name',
+                     'children': []},
+                    {'name': 'child_name3',
+                     'parent': 'parent_name',
+                     'children': [
+                         {'name': 'child_child',
+                          'parent': 'child_name3',
+                          'children': []}]},
+                     ]}]
+        result_function = tree_from_list(
+            rows,
+            id_field='name',
+            parent_field='parent',
+            children_field='children',
+            root_parent=None)
+        self.assertEqual(result_function, result_good)
+
+    def test_tree_from_list_cyclic(self):
+        """
+        Cycle detection 1
+        """
+        rows = [{'name': 'child_name', 'parent': 'parent_name'},
+                {'name': 'parent_name', 'parent': 'child_name'}]
+        self.assertRaises(
+            CycleError,
+            tree_from_list,
+            rows,
+            id_field='name',
+            parent_field='parent',
+            children_field='children',
+            root_parent='parent_name')
+
+    def test_tree_from_list_cyclic2(self):
+        """
+        Cycle detection 2
+        """
+        rows = [{'name': 'parent_name', 'parent': 'parent_name'}, ]
+        self.assertRaises(
+            CycleError,
+            tree_from_list,
+            rows,
+            id_field='name',
+            parent_field='parent',
+            children_field='children',
+            root_parent='child_name')
+
+    def test_named_list(self):
+        rows = [
+            ['a', 'b', 'c', 'd'],
+            ['f', 'g', 'h', 'i']]
+        names = ['name1', 'name2', 'name3', 'name4']
+        result = named_list(rows, names)
+        result_good = [
+            {'name1': 'a', 'name2': 'b', 'name3': 'c', 'name4': 'd'},
+            {'name1': 'f', 'name2': 'g', 'name3': 'h', 'name4': 'i'}]
+        self.assertEqual(result, result_good)
+
+    def test_unique_list(self):
+        rows = [1, 2, 2, 3, 4, 5, 5, 7, 2, 5]
+        result = unique_list(rows)
+        result_good = [1, 2, 3, 4, 5, 7]
+        self.assertEqual(result, result_good)
+
+    def test_unique_list2(self):
+        rows = [[1, 2], [2, 2], [3, 4], [2, 2], [1, 2]]
+        result = unique_list(rows)
+        result_good = [[1, 2], [2, 2], [3, 4]]
+        self.assertEqual(result, result_good)
+
+
+class CoordinatesTest(TestCase):
+
+    def test_detect_prj1(self):
+        prj = None
+        self.assertEquals(
+            lizard_map.coordinates.detect_prj(prj),
+            lizard_map.coordinates.RD)
+
+    def test_detect_prj2(self):
+        prj = ('PROJCS["RD_New",GEOGCS["GCS_Amersfoort",'
+               'DATUM["D_Amersfoort",'
+               'SPHEROID["Bessel_1841",6377397.155,299.1528128]],'
+               'PRIMEM["Greenwich",0.0],'
+               'UNIT["Degree",0.0174532925199433]],'
+               'PROJECTION["Double_Stereographic"],'
+               'PARAMETER["False_Easting",155000.0],'
+               'PARAMETER["False_Northing",463000.0],'
+               'PARAMETER["Central_Meridian",5.38763888888889],'
+               'PARAMETER["Scale_Factor",0.9999079],'
+               'PARAMETER["Latitude_Of_Origin",52.15616055555555],'
+               'UNIT["Meter",1.0]]')
+        self.assertEquals(
+            lizard_map.coordinates.detect_prj(prj),
+            lizard_map.coordinates.RD)
+
+    def test_detect_prj3(self):
+        prj = ('GEOGCS["GCS_WGS_1984",'
+               'DATUM["D_WGS_1984",'
+               'SPHEROID["WGS_1984",6378137.0,298.257223563]],'
+               'PRIMEM["Greenwich",0.0],'
+               'UNIT["Degree",0.0174532925199433]]')
+        self.assertEquals(
+            lizard_map.coordinates.detect_prj(prj),
+            lizard_map.coordinates.WGS84)
