@@ -1,5 +1,6 @@
 import StringIO
 
+from django.conf import settings
 from django.db.models import Max
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404
@@ -9,6 +10,7 @@ from django.utils import simplejson as json
 from django.views.decorators.cache import never_cache
 import Image
 import csv
+import logging
 import mapnik
 
 from lizard_map import coordinates
@@ -22,12 +24,45 @@ from lizard_map.models import WorkspaceItem
 from lizard_map.models import WorkspaceCollage
 from lizard_map.models import WorkspaceCollageSnippet
 from lizard_map.models import WorkspaceCollageSnippetGroup
+from lizard_map.utility import analyze_http_user_agent
 from lizard_map.utility import short_string
 from lizard_map.workspace import WorkspaceManager
 
 # Workspace stuff
 
 CUSTOM_LEGENDS = 'custom_legends'
+MAP_LOCATION = 'map_location'
+CRUMBS_HOMEPAGE = {'name': 'home', 'title': 'hoofdpagina', 'url': '/'}
+
+logger = logging.getLogger(__name__)
+
+
+def homepage(request,
+             template='lizard_map/example_homepage.html',
+             crumbs_prepend=None):
+    """Default apps screen, make your own template.
+    """
+    date_range_form = DateRangeForm(
+        current_start_end_dates(request, for_form=True))
+
+    workspace_manager = WorkspaceManager(request)
+    workspaces = workspace_manager.load_or_create()
+    date_range_form = DateRangeForm(
+        current_start_end_dates(request, for_form=True))
+
+    if crumbs_prepend is not None:
+        crumbs = list(crumbs_prepend)
+    else:
+        crumbs = [CRUMBS_HOMEPAGE]
+
+    return render_to_response(
+        template,
+        {'date_range_form': date_range_form,
+         'workspaces': workspaces,
+         'javascript_hover_handler': 'popup_hover_handler',
+         'javascript_click_handler': 'popup_click_handler',
+         'crumbs': crumbs},
+        context_instance=RequestContext(request))
 
 
 def workspace(request,
@@ -117,6 +152,18 @@ def workspace_item_add(request,
     return HttpResponse(json.dumps(workspace.id))
 
 
+@never_cache
+def workspace_item_empty(request,
+                       workspace_id,
+                       is_temp_workspace=False,
+                       template='lizard_map/tag_workspace.html'):
+    """Clear workspace items for given workspace."""
+    workspace = get_object_or_404(Workspace, pk=workspace_id)
+    workspace.workspace_items.all().delete()
+
+    return HttpResponse("")
+
+
 @never_cache
 def workspace_item_edit(request, workspace_item_id=None, visible=None):
     """edits a workspace_item
@@ -139,6 +186,15 @@ def workspace_item_edit(request, workspace_item_id=None, visible=None):
     return HttpResponse(json.dumps(workspace_item.workspace.id))
 
 
+@never_cache
+def workspace_item_extent(request, workspace_item_id=None):
+    """Returns extent for the workspace in json.
+    """
+    workspace_item_id = request.GET['workspace_item_id']
+    workspace_item = get_object_or_404(WorkspaceItem, pk=workspace_item_id)
+    return HttpResponse(json.dumps(workspace_item.adapter.extent()))
+
+
 @never_cache
 def snippet_group_graph_edit(request, snippet_group_id):
     """Edits snippet_group properties using post.
@@ -265,12 +321,12 @@ def workspace_item_delete(request, object_id=None):
 def session_workspace_edit_item(request,
                                 workspace_item_id=None,
                                 workspace_category='user'):
-    """edits workspace item, the function automatically finds best appropriate
-    workspace
+    """edits workspace item, the function automatically finds best
+    appropriate workspace
 
-    if workspace_item_id is None, a new workspace_item will be created using
-    workspace_item_add TODO if workspace_item_id is filled in, apply edits and
-    save
+    if workspace_item_id is None, a new workspace_item will be created
+    using workspace_item_add TODO if workspace_item_id is filled in,
+    apply edits and save
 
     """
     workspace_id = request.session['workspaces'][workspace_category][0]
@@ -285,6 +341,18 @@ def session_workspace_edit_item(request,
     return
 
 
+@never_cache
+def session_workspace_extent(request, workspace_category='user'):
+    """Returns extent for the workspace in json.
+    """
+    if 'workspaces' in request.session:
+        workspace_id = request.session['workspaces'][workspace_category][0]
+        workspace = get_object_or_404(Workspace, pk=workspace_id)
+        return HttpResponse(json.dumps(workspace.extent()))
+    else:
+        return HttpResponse(json.dumps(''))
+
+
 def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
     """Return html with info on list of 'found' objects.
 
@@ -355,7 +423,8 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
                             'legend': True},
             )
 
-        x_found, y_found = display_object['google_coords']
+        if 'google_coords' in display_object:
+            x_found, y_found = display_object['google_coords']
         result_html += html_per_workspace_item
 
     if popup_id is None:
@@ -372,7 +441,6 @@ def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
 def popup_collage_json(collage, popup_id, request=None):
     """
     display collage by adding display_groups together
-    TODO: must work with items from same adapter, not only workspace_item
     """
 
     result_html = ''
@@ -392,8 +460,10 @@ def popup_collage_json(collage, popup_id, request=None):
                 snippet_group=snippet_group,
                 layout_options={'legend': True})
             result_html += html_per_workspace_item
+
             # Pick the location of the first snippet.
-            google_x, google_y = snippets[0].location['google_coords']
+            if 'google_coords' in snippets[0].location:
+                google_x, google_y = snippets[0].location['google_coords']
 
     result = {'id': popup_id,
               'x': google_x,
@@ -415,9 +485,12 @@ def collage(request,
     date_range_form = DateRangeForm(
         current_start_end_dates(request, for_form=True))
     show_table = request.GET.get('show_table', False)
+
+    collage = get_object_or_404(WorkspaceCollage, pk=collage_id)
+
     return render_to_response(
         template,
-        {'collage': get_object_or_404(WorkspaceCollage, pk=collage_id),
+        {'collage': collage,
          'editable': editable,
          'date_range_form': date_range_form,
          'request': request,
@@ -647,7 +720,9 @@ def wms(request, workspace_id):
     mapnik_map.background = mapnik.Color('transparent')
     #m.background = mapnik.Color('blue')
 
-    for workspace_item in workspace.workspace_items.filter(visible=True):
+    workspace_items = workspace.workspace_items.filter(visible=True).reverse()
+    for workspace_item in workspace_items:
+        logger.debug("Drawing layer for %s..." % workspace_item)
         layers, styles = workspace_item.adapter.layer(layer_ids=layers,
                                                       request=request)
         layers.reverse()  # first item should be drawn on top (=last)
@@ -655,13 +730,18 @@ def wms(request, workspace_id):
             mapnik_map.layers.append(layer)
         for name in styles:
             mapnik_map.append_style(name, styles[name])
+        # mapnik_map.zoom_to_box(layer.envelope())
 
     #Zoom and create image
+    logger.debug("Zooming to box...")
     mapnik_map.zoom_to_box(mapnik.Envelope(*bbox))
-    # m.zoom_to_box(layer.envelope())
+    # mapnik_map.zoom_to_box(layer.envelope())
     img = mapnik.Image(width, height)
+    logger.debug("Rendering map...")
     mapnik.render(mapnik_map, img)
     http_user_agent = request.META.get('HTTP_USER_AGENT', '')
+
+    logger.debug("Converting image to rgba...")
     rgba_image = Image.fromstring('RGBA', (width, height), img.tostring())
     buf = StringIO.StringIO()
     if 'MSIE 6.0' in http_user_agent:
@@ -726,6 +806,13 @@ def search_coordinates(request):
     y = float(request.GET.get('y'))
     # TODO: convert radius to correct scale (works now for google + rd)
     radius = float(request.GET.get('radius'))
+    radius_search = radius
+    if 'HTTP_USER_AGENT' in request.META:
+        analyzed_user_agent = analyze_http_user_agent(
+            request.META['HTTP_USER_AGENT'])
+        # It's more difficult to point with your finger than with the mouse.
+        if analyzed_user_agent['device'] == 'iPad':
+            radius_search = radius_search * 3
     srs = request.GET.get('srs')
     google_x, google_y = coordinates.srs_to_google(srs, x, y)
 
@@ -735,7 +822,7 @@ def search_coordinates(request):
             for workspace_item in workspace.workspace_items.filter(
                 visible=True):
                 search_results = workspace_item.adapter.search(
-                    google_x, google_y, radius=radius)
+                    google_x, google_y, radius=radius_search)
                 found += search_results
     if found:
         # ``found`` is a list of dicts {'distance': ..., 'timeserie': ...}.
@@ -827,3 +914,42 @@ def export_snippet_group_statistics_csv(request, snippet_group_id=None):
     for row in statistics:
         writer.writerow([row[colname] for colname in colnames])
     return response
+
+
+"""
+Map locations are stored in the session with key MAP_SESSION. It
+contains a dictionary with fields x, y and zoom.
+"""
+
+
+def map_location_save(request):
+    """
+    Saves given coordinates as strings (POST x, y, zoom) to session.
+    """
+    x = request.POST['x']
+    y = request.POST['y']
+    zoom = request.POST['zoom']
+    request.session[MAP_LOCATION] = {'x': x, 'y': y, 'zoom': zoom}
+    return HttpResponse("")
+
+
+def map_location_load_default(request):
+    """
+    Returns stored coordinates in a json dict, or empty dict if
+    nothing was saved.
+    """
+    try:
+        map_settings = settings.MAP_SETTINGS
+        x = map_settings['startlocation_x']
+        y = map_settings['startlocation_y']
+        zoom = map_settings['startlocation_zoom']
+    except AttributeError:
+        logger.warn(
+            'Could not find MAP_SETTINGS in '
+            'django settings or MAP_SETTINGS '
+            'not properly configured, using default coordinates.')
+        x, y, zoom = '550000', '6850000', '10'
+
+    map_location = {'x': x, 'y': y, 'zoom': zoom}
+
+    return HttpResponse(json.dumps(map_location))
